.\" Emacs, this is in -*- nroff -*- mode "
.TH dragdrop n "19th July 1995" "DragDrop 2.3" "Tcl/Tk Source Extensions"
.SH NAME
dragdrop \- Drag-and-drop support for Tcl7.4/Tk4.0
.SH "PROCEDURE SYNOPSIS"
.nf
.BI dragdrop_bind " widget data_server generator button ?notifier?"
.BI dragdrop_destination " widget drag_handler types drop_handler"

.BI supplies_string " widget string"
.BI supply_string " widget type"
.BI accepts_drop " x y source_app data_type_list"

.BI drag_start " x y types"
.BI drag " x y"
.BI drop " x y"

.BI drag_at " x y source_app"
.BI drop_at " x y source_app"

.BI drop_get_data " type"
.BI dragdrop_find " x y"
.fi
.SH "VARIABLE SYNOPSIS"
.nf
.B dragdrop
.B dragdropnotify
.B dragserver
.B draghandler
.B drophandler
.B dropacceptedtypes
.fi
.SH DESCRIPTION
This is a Tcl-level library (actually a combination of two libraries)
for inter- and intra-interpreter drag-and-drop for Tk4.0 applications.

.B dragdrop_bind
and
.B dragdrop_destination
implement the main user interface to the drag-and-drop code, though
for complex bindings it might be necessary to access
.B drag_start,
.B drag
and
.B drop
directly.
.B drag_at,
.B drop_at
and
.B drop_get_data
are designed to only be called by
.B send
from a remote Tk application, and are not supposed to be called by
users of the library unless they are reimplementing part of the
library.

.B supply_string,
.B supplies_string
and
.B accepts_drop
are sample procedures for the drag-and-drop servers and control
routines, and their use is demonstrated in
.BR dragdrop_demo (1).

.B dragdrop_find
implements a search algorithm, finding which interpreter (and which
widget within that interpreter) actually has the mouse cursor in it.
It uses cached values of the interpreter to reduce the amount of IPC
for the average case (motion within an interpreter).

.B dragdrop,
.B dragdropnotify,
.B dragserver,
.B draghandler,
.B drophandler
and
.B dropacceptedtypes
are arrays used internally to the drag-and-drop system. Mostly, these
should not be modified by the user (except where duplicating the
action of
.B dragdrop_bind
or
.B dragdrop_server ),
but there are three user-modifiable members of the
.B dragdrop
array:
.TP
.B "dragdrop(debug)"
Set to 1 to turn on debugging
.RB ( NB
this is rather verbose). This is intended to be turned on by a remote
application, as this is how the author uses it...
.TP
.B "dragdrop(yes)"
The cursor used when it is possible to drop data at the current
position, in any form acceptable to 
.BR Tk_GetCursor (3)
.TP
.B "dragdrop(no)"
The cursor used when it is not possible to drop data at the current
position, in any form acceptable to 
.BR Tk_GetCursor (3)
.SH "CREATING A DRAG-AND-DROP SERVER"
The command
.RS

.BI dragdrop_bind " widget data_server generator button ?notifier?"

.RE
makes the widget described by the
.I widget
argument a source of drag-and-drop data. The other arguments are:
.TP
.I "data_server"
Specifies a procedure to call to supply the data requested when during
a drop. This takes the arguments:
.RS
.TP
.I widget
the source widget
.TP
.I type
type of data requested
.RE
.TP
.I "generator"
This is a script to evaluate at the start of a drag operation which
informs the dragging engine of the types of data which the data_server
is capable of supplying. It can also be used to move the relevant
parts of the state of the source widget into a temporary buffer. See
.BR dragdrop_types (7)
for details of the type scheme.
.TP
.I "button"
This is which mouse button the dragging operation should be bound
to. Ideally this would be button 1, but this tends to conflict with
other bindings on the widget, so great care should be exercised where
such a tactic is adopted.
.TP
.I "notifier"
This is the name of a procedure to call to signify the progress of the
current dragging operation (defaults to none). The first argument
indicates what sort of message is being passed. The actual forms
generated are:
.RS
.TP
.IB notifier " startat " "widget x y"
This call is generated when the mouse button is pressed to begin the
drag operation, and could be used to bring up a window indicating the
data being carried by the drag operation.
.TP
.IB notifier " able " "widget x y drop_interpreter drop_widget"
This call is generated when a drop operation would be possible at the
.I drop_widget
given in the
.I drop_interpreter.
This is only generated from mouse movement with the button down.
.TP
.IB notifier " unable " "widget x y"
This call is generated when a drop operation would not be bossible at
the current location (either because you are not over a Tk4
application, or the widget you are currently over does not support
dropping at the current location in the destination
widget/interpreter). This is only generated from mouse movement with
the button down.
.TP
.IB notifier " dropped " "widget x y drop_interpreter drop_widget"
This call is generated when the data has been successfully dropped
into the
.I drop_interpreter
at the location 
.I x,y
which lies within the 
.I drop_widget.
This can be used to withdraw or destroy any windows created by the
.B startat 
call and to also delete the data from the source widget if this is so
required. This is only generated by a release of the dragging button.
.TP
.IB notifier " failed " "widget x y"
This call is generated when the drag-and-drop engine failed to
transfer the data from the source to any destination, and is called
when the dragging button is released. It can be used to delete or
withdraw any windows created by the 
.B startat
call.
.RE

.PP
An example usage of the
.B dragdrop_bind
command is:
.RS
.nf
\fCpack [entry .e]
dragdrop_bind .e supply_string {
	supplies_string .e [.e get]
} 3\fR
.fi
.RE
.PP
This creates an entry widget and makes it into a source for
drag-and-drop operations. It only supplies data of type
.I STRING
where the data supplied is the contents of the entry, as supplied by
the  .e get  widget command.
.SH "CREATING A DRAG-AND-DROP CLIENT"
The command
.RS

.BI dragdrop_destination " widget drag_handler types drop_handler"

.RE
makes the widget described by the
.I "widget"
argument a destination for drag-and-drop data. The other arguments
are:
.TP
.I "drag_handler"
This is a procedure which returns a boolean value (as determined by
.BR Tcl_GetBoolean (3))
stating whether a drop operation is possible at the location given
within the widget, this information being used to select the cursor
used, so relaying whether a drop would succeed to the user. The
procedure is called as:

.RS
.RS
.I "handler widget x y source_interpreter data_types"
.RE
.RE

.RS
The arguments passed are:
.TP
.I widget
The widget underneath the mouse cursor.
.TP
.I x y
The coordinates of the mouse cursor. Unlike the coordinates used
throughout the rest of the drag-and-drop engine, these are relative to
the destination widget. If these are empty strings then the mouse
cursor has left the widget.
.TP
.I source_interpreter
The interpreter which will supply the data to be dropped.
.B NB
The data is not necessarily available at this point.
.TP
.I data_types
List of types of data that the
.I source_interpreter
is prepared to supply the data as.
.RE

.TP
.I "types"
This is a list of the types accepted by the 
.I drop_handler 
of the widget in order of preference. The drag-and-drop mechanism
selects the first member of this list that the data source is prepared
to supply and retrieves the data associated to this type from the
source. Some suggested types are listed elsewhere in this document.
.TP
.I "drop_handler"
This is a procedure which performs the actual handling of the dropped
data. It is called as:

.RS
.RS
.I "handler widget x y source_interpreter data_type data"
.RE
.RE

.RS
The arguments passed are:
.TP
.I widget
The widget underneath the mouse cursor.
.TP
.I x y
The coordinates of the mouse cursor. Unlike the coordinates used
throughout the rest of the drag-and-drop engine, these are relative to
the destination widget. If these are empty strings then the mouse
cursor has left the widget.
.TP
.I source_interpreter
The interpreter that supplied the data dropped.
.TP
.I data_type
The type of the data dropped.
.TP
.I data
The data dropped.
.B NB
This data is not necessarily still available in the
.I source_interpreter
.RE
.PP
An example usage of the
.B dragdrop_destination
command is:
.RS
.nf
\fCproc drop_on_listbox {w x y s t d} {
	insert end {}
	insert "@$x,$y" $d
	delete end
}
pack [listbox .l]
dragdrop_destination .l accepts_drop STRING drop_on_listbox\fR
.fi
.RE
.PP
This creates an entry widget and makes it into a source for
drag-and-drop operations. It only supplies data of type
.I STRING
where the data supplied is the contents of the entry, as supplied by
the  .e get  widget command.

.SH "DESCRIPTION OF VARIABLES"
The drag-and-drop system uses six arrays:
.TP
.B dragdrop
This contains the main state of the system, with the following fields:

.RS
.TP
.I debug
debugging flag.
.TP
.I yes
cursor (in form acceptable to 
.BR Tk_GetCursor (3))
to use when a drop is possible. Defaults to a green arrow.
.TP
.I no
cursor (in form acceptable to 
.BR Tk_GetCursor (3))
to use when a drop is not possible. Defaults to a red circle.
.TP
.I types
possible types of data to transfer.
.TP
.I source
source interpreter (in destination).
.TP
.I widget
source widget.
.TP
.I cursor
original cursor used in source widget.
.TP
.I rmt_widget
destination widget (used for caching)
.TP
.I rmt_interp
destination interpreter (used for caching)
.TP
.I "supplies,*"
temporaries used by
.B supply_string
and
.B supplies_string
.RE
.TP
.B dragdropnotify
This array (indexed by widget name) is used to store action notify
procedure names for the current interpreter (part of the drag-and-drop
source side).

.TP
.B dragserver
This array (indexed by widget name) is used to store data generation
procedure names for the current interpreter (part of the drag-and-drop
source side).

.TP
.B draghandler
This array (indexed by widget name) is used to store drag action
handler procedure names for the current interpreter (part of the
drag-and-drop destination side).

.TP
.B drophandler
This array (indexed by widget name) is used to store drop action
handler procedure names for the current interpreter (part of the
drag-and-drop destination side).

.TP
.B dropacceptedtypes
This array (indexed by widget name) is used to store lists of types
accepted by the drag-and-drop destination procedures (specifically the
drop action handlers).
.SH FILES
.TP
.I dragdrop.tcl
Implements the drag-and-drop protocol (procedures
.BR drag_start ,
.BR drag ,
.BR drop ,
.BR drop_get_data ,
.BR drag_at ", and"
.BR drop_at )
.TP
.I dragdrop_ifc.tcl
Implements a sample programmers interface to the drag-and-drop
protocol (procedures
.BR dragdrop_bind ,
.BR dragdrop_destination ,
.BR supplies_string ,
.BR supply_string ", and"
.BR accepts_drop )
.TP
.I tclIndex
Tcl autoload directory 
.SH "SEE ALSO"
.BR Tcl_GetBoolean (3),
.BR Tk_GetCursor (3),
.BR dragdrop (7),
.BR dragdrop_demo (1),
.BR dragdrop_types (7),
.BR send (n),
.BR winfo (n),
.BR wm (n)
.SH BUGS
Are you kidding? Mind you, I'd like to know of any bugs that you find,
both in the library and in this documentation.
.SH "TO DO"
Document the actual drag-and-drop protocol.

Add hooks to allow (with a suitable C extension) dragging to a Motif
application. Allowing dragging in the opposite direction would be
tricky though.

Make a real application that uses this package.
.SH AUTHOR
Donal K. Fellows, \fCfellowsd@cs.man.ac.uk\fR
